@@ -1,7 +1,21 @@
 import datetime
 import logging
+from pathlib import Path
 
-LOG_LEVEL = logging.INFO
+DEV_MODE = False  ## updates AST output each run
+
+if DEV_MODE:
+    print("""\
+
+
+
+    In DEV_MODE
+
+
+
+    """)
+
+LOG_LEVEL = logging.DEBUG if DEV_MODE else logging.INFO
 
 ## When testing user-supplied snippets watch out for the BOM MS inserts via Notepad. AST chokes on it.
 
@@ -12,9 +26,23 @@ def sorted(my_list):
 """
 
 TEST_SNIPPET = """\
-def sorted(my_list):
-    sorted_list = my_list.sort()
-    return sorted_list
+fruit = ['banana']
+try:
+    lunch = fruit[100]
+except (IndexError, TypeError):
+    print("No lunch for you!")
+except Exception as e:
+    print(f"Unknown error - details: {e}")
+
+try:
+    float('boat')
+except ValueError:
+    print("You can't float a boat! Only a number of some sort!")
+
+try:
+    names[100]
+except Exception:
+    print(names)
 """
 
 DEMO_SNIPPET = """\
@@ -140,8 +168,7 @@ def camelCase(a, b, c, d, f, *, g):
     pass
 """
 
-DEV_MODE = (LOG_LEVEL == logging.DEBUG)  ## updates AST output each run
-AST_OUTPUT_XML = 'ast_output.xml'
+AST_OUTPUT_XML = Path(__file__).parent / 'ast_output.xml'
 
 PYTHON_CODE_START = '__python_code_start__'
 PYTHON_CODE_END = '__python_code_end__'
